@@ -1,13 +1,11 @@
 #!/usr/bin/env python2
-
-'''
-You should normally never use this! Use emcc instead.
+"""You should normally never use this! Use emcc instead.
 
 This is a small wrapper script around the core JS compiler. This calls that
 compiler with the settings given to it. It can also read data from C/C++
 header files (so that the JS compiler can see the constants in those
 headers, for the libc implementation in JS).
-'''
+"""
 
 from tools.toolchain_profiler import ToolchainProfiler
 if __name__ == '__main__':
@@ -20,7 +18,7 @@ from collections import OrderedDict
 
 from tools import shared
 from tools import jsrun, cache as cache_module, tempfiles
-from tools.response_file import read_response_file
+from tools.response_file import substitute_response_files
 from tools.shared import WINDOWS, asstr
 
 __rootpath__ = os.path.abspath(os.path.dirname(__file__))
@@ -61,8 +59,8 @@ def access_quoter(settings):
   return access_quote
 
 
-def emscript(infile, settings, outfile, libraries=None, compiler_engine=None,
-             temp_files=None, DEBUG=None):
+def emscript(infile, settings, outfile, libraries, compiler_engine, temp_files,
+             DEBUG):
   """Runs the emscripten LLVM-to-JS compiler.
 
   Args:
@@ -72,8 +70,6 @@ def emscript(infile, settings, outfile, libraries=None, compiler_engine=None,
     outfile: The file where the output is written.
   """
 
-  if libraries is None: libraries = []
-
   assert settings['ASM_JS'], 'fastcomp is asm.js-only (mode 1 or 2)'
 
   success = False
@@ -147,7 +143,7 @@ def parse_backend_output(backend_output, DEBUG):
   mem_init = mem_init.replace('Runtime.', '')
 
   try:
-    #if DEBUG: print >> sys.stderr, "METAraw", metadata_raw
+    #if DEBUG: logging.debug("METAraw %s", metadata_raw)
     metadata = json.loads(metadata_raw, object_pairs_hook=OrderedDict)
   except Exception as e:
     logging.error('emscript: failure to parse metadata output from compiler backend. raw output is: \n' + metadata_raw)
@@ -161,14 +157,19 @@ def parse_backend_output(backend_output, DEBUG):
 
 def fixup_metadata_tables(metadata, settings):
   # if emulating pointer casts, force all tables to the size of the largest
-  if settings['EMULATE_FUNCTION_POINTER_CASTS']:
+  # (for wasm, we use binaryen's fpcast-emu pass, we don't need to do anything
+  # here)
+  if settings['EMULATE_FUNCTION_POINTER_CASTS'] and not settings['WASM']:
     max_size = 0
     for k, v in metadata['tables'].items():
       max_size = max(max_size, v.count(',')+1)
     for k, v in metadata['tables'].items():
       curr = v.count(',')+1
       if curr < max_size:
-        metadata['tables'][k] = v.replace(']', (',0'*(max_size - curr)) + ']')
+        if v.count('[]') == 1:
+          metadata['tables'][k] = v.replace(']', (','.join(['0'] * (max_size - curr)) + ']'))
+        else:
+          metadata['tables'][k] = v.replace(']', (',0'*(max_size - curr)) + ']')
 
   if settings['SIDE_MODULE']:
     for k in metadata['tables'].keys():
@@ -322,7 +323,7 @@ def function_tables_and_exports(funcs, metadata, mem_init, glue, forwarded_data,
 
   function_tables_impls = make_function_tables_impls(function_table_data, settings)
   final_function_tables = '\n'.join(function_tables_impls) + '\n' + function_tables_defs
-  if settings.get('EMULATED_FUNCTION_POINTERS'):
+  if settings['EMULATED_FUNCTION_POINTERS']:
     final_function_tables = (
       final_function_tables
       .replace("asm['", '')
@@ -372,7 +373,7 @@ def create_module(function_table_sigs, metadata, settings,
   asm_start = asm_start_pre + '\n' + asm_global_vars + asm_temp_vars + asm_runtime_thread_local_vars + '\n' + asm_global_funcs
 
   temp_float = '  var tempFloat = %s;\n' % ('Math_fround(0)' if provide_fround(settings) else '0.0')
-  async_state = '  var asyncState = 0;\n' if settings.get('EMTERPRETIFY_ASYNC') else ''
+  async_state = '  var asyncState = 0;\n' if settings['EMTERPRETIFY_ASYNC'] else ''
   f0_fround = '  const f0 = Math_fround(0);\n' if provide_fround(settings) else ''
 
   replace_memory = create_replace_memory(settings)
@@ -519,17 +520,21 @@ def update_settings_glue(settings, metadata):
   settings['MAX_GLOBAL_ALIGN'] = metadata['maxGlobalAlign']
   settings['IMPLEMENTED_FUNCTIONS'] = metadata['implementedFunctions']
 
+  # addFunction support for Wasm backend
+  if settings['WASM_BACKEND'] and settings['RESERVED_FUNCTION_POINTERS'] > 0:
+    start_index = metadata['jsCallStartIndex']
+    # e.g. jsCallFunctionType ['v', 'ii'] -> sig2order{'v': 0, 'ii': 1}
+    sig2order = {sig: i for i, sig in enumerate(metadata['jsCallFuncType'])}
+    # Index in the Wasm function table in which jsCall thunk function starts
+    settings['JSCALL_START_INDEX'] = start_index
+    settings['JSCALL_SIG_ORDER'] = sig2order
+
 
 def compile_settings(compiler_engine, settings, libraries, temp_files):
   # Save settings to a file to work around v8 issue 1579
   with temp_files.get_file('.txt') as settings_file:
-    def save_settings():
-      global settings_text
-      settings_text = json.dumps(settings, sort_keys=True)
-      s = open(settings_file, 'w')
-      s.write(settings_text)
-      s.close()
-    save_settings()
+    with open(settings_file, 'w') as s:
+      json.dump(settings, s, sort_keys=True)
 
     # Call js compiler
     out = jsrun.run_js(path_from_root('src', 'compiler.js'), compiler_engine,
@@ -541,7 +546,7 @@ def compile_settings(compiler_engine, settings, libraries, temp_files):
 
 
 def memory_and_global_initializers(pre, metadata, mem_init, settings):
-  global_initializers = str(', '.join(['{ func: function() { %s() } }' % i for i in metadata['initializers']]))
+  global_initializers = ', '.join('{ func: function() { %s() } }' % i for i in metadata['initializers'])
 
   if settings['SIMD'] == 1:
     pre = open(path_from_root(os.path.join('src', 'ecmascript_simd.js'))).read() + '\n\n' + pre
@@ -608,7 +613,7 @@ def get_original_exported_functions(settings):
 
 
 def check_all_implemented(all_implemented, pre, settings):
-  if settings['ASSERTIONS'] and settings.get('ORIGINAL_EXPORTED_FUNCTIONS'):
+  if settings['ASSERTIONS'] and settings['ORIGINAL_EXPORTED_FUNCTIONS']:
     original_exports = get_original_exported_functions(settings)
     for requested in original_exports:
       if not is_already_implemented(requested, pre, all_implemented):
@@ -647,7 +652,7 @@ def get_exported_implemented_functions(all_exported_functions, all_implemented,
     if settings['EMTERPRETIFY']:
       funcs += ['emterpret']
       if settings['EMTERPRETIFY_ASYNC']:
-        funcs += ['setAsyncState', 'emtStackSave', 'emtStackRestore']
+        funcs += ['setAsyncState', 'emtStackSave', 'emtStackRestore', 'getEmtStackMax', 'setEmtStackMax']
     if settings['ASYNCIFY']:
       funcs += ['setAsync']
 
@@ -694,8 +699,11 @@ function _emscripten_asm_const_%s(%s) {
   asm_consts_text = '\nvar ASM_CONSTS = [' + ',\n '.join(asm_consts) + '];\n'
   asm_funcs_text = '\n'.join(asm_const_funcs) + '\n'
 
+  em_js_funcs = create_em_js(forwarded_json, metadata)
+  em_js_text = '\n'.join(em_js_funcs) + '\n'
+
   body_marker = '// === Body ==='
-  return pre.replace(body_marker, body_marker + '\n' + asm_consts_text + asstr(asm_funcs_text))
+  return pre.replace(body_marker, body_marker + '\n' + asm_consts_text + asstr(asm_funcs_text) + em_js_text)
 
 # Test if the parentheses at body[openIdx] and body[closeIdx] are a match to each other.
 def parentheses_match(body, openIdx, closeIdx):
@@ -760,17 +768,16 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
   def make_coercions(sig): return ';'.join(['p%d = %s' % (p, shared.JS.make_coercion('p%d' % p, sig[p+1], settings)) for p in range(len(sig)-1)]) + ';'
 
   # when emulating function pointer casts, we need to know what is the target of each pointer
-  if settings['EMULATE_FUNCTION_POINTER_CASTS']:
+  if settings['EMULATE_FUNCTION_POINTER_CASTS'] and not settings['WASM']:
     function_pointer_targets = {}
     for sig, table in function_table_data.items():
       start = table.index('[')
       end = table.rindex(']')
       body = table[start+1:end].split(',')
-      parsed = [x.strip() for x in body]
-      for i in range(len(parsed)):
-        if parsed[i] != '0':
+      for i, parsed in enumerate(x.strip() for x in body):
+        if parsed != '0':
           assert i not in function_pointer_targets
-          function_pointer_targets[i] = [sig, str(parsed[i])]
+          function_pointer_targets[i] = [sig, str(parsed)]
 
   def make_table(sig, raw):
     if '[]' in raw: return ('', '') # empty table
@@ -809,7 +816,7 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
       body = list(map(receive, body))
     for j in range(settings['RESERVED_FUNCTION_POINTERS']):
       curr = 'jsCall_%s_%s' % (sig, j)
-      body[settings['FUNCTION_POINTER_ALIGNMENT'] * (1 + j)] = curr
+      body[1 + j] = curr
       implemented_functions.add(curr)
     Counter.next_item = 0
     def fix_item(item):
@@ -817,7 +824,7 @@ def make_function_tables_defs(implemented_functions, all_implemented, function_t
       Counter.next_item += 1
       newline = Counter.next_item % 30 == 29
       if item == '0':
-        if j > 0 and settings['EMULATE_FUNCTION_POINTER_CASTS'] and j in function_pointer_targets: # emulate all non-null pointer calls, if asked to
+        if j > 0 and settings['EMULATE_FUNCTION_POINTER_CASTS'] and not settings['WASM'] and j in function_pointer_targets: # emulate all non-null pointer calls, if asked to
           proper_sig, proper_target = function_pointer_targets[j]
           if settings['EMULATED_FUNCTION_POINTERS']:
             if proper_target in all_implemented:
@@ -925,8 +932,8 @@ function jsCall_%s_%s(%s) {
 
 def create_mftCall_funcs(function_table_data, settings):
   mftCall_funcs = []
-  if settings.get('EMULATED_FUNCTION_POINTERS'):
-    if settings.get('RELOCATABLE') and not settings['BINARYEN']: # in wasm, emulated function pointers are just simple table calls
+  if settings['EMULATED_FUNCTION_POINTERS']:
+    if settings['RELOCATABLE'] and not settings['BINARYEN']: # in wasm, emulated function pointers are just simple table calls
       for sig, table in function_table_data.items():
         params = ','.join(['ptr'] + ['p%d' % p for p in range(len(sig)-1)])
         coerced_params = ','.join([shared.JS.make_coercion('ptr', 'i', settings)] + [shared.JS.make_coercion('p%d', unfloat(sig[p+1]), settings) % p for p in range(len(sig)-1)])
@@ -1013,10 +1020,7 @@ def global_simd_funcs(access_quote, metadata, settings):
     return ''
 
   def string_contains_any(s, str_list):
-    for sub in str_list:
-      if sub in s:
-        return True
-    return False
+    return any(sub in s for sub in str_list)
 
   nonexisting_simd_symbols = ['Int8x16_fromInt8x16', 'Uint8x16_fromUint8x16', 'Int16x8_fromInt16x8', 'Uint16x8_fromUint16x8', 'Int32x4_fromInt32x4', 'Uint32x4_fromUint32x4', 'Float32x4_fromFloat32x4', 'Float64x2_fromFloat64x2']
   nonexisting_simd_symbols += ['Int32x4_addSaturate', 'Int32x4_subSaturate', 'Uint32x4_addSaturate', 'Uint32x4_subSaturate']
@@ -1173,7 +1177,7 @@ def create_asm_setup(debug_tables, function_table_data, metadata, settings):
 
   asm_setup += setup_function_pointers(function_table_sigs, settings)
 
-  if settings.get('EMULATED_FUNCTION_POINTERS'):
+  if settings['EMULATED_FUNCTION_POINTERS']:
     function_tables_impls = make_function_tables_impls(function_table_data, settings)
     asm_setup += '\n' + '\n'.join(function_tables_impls) + '\n'
 
@@ -1184,19 +1188,32 @@ def setup_function_pointers(function_table_sigs, settings):
   asm_setup = ''
   for sig in function_table_sigs:
     asm_setup += '\n' + shared.JS.make_invoke(sig) + '\n'
-    if settings.get('RESERVED_FUNCTION_POINTERS'):
+    if settings['RESERVED_FUNCTION_POINTERS']:
       asm_setup += '\n' + shared.JS.make_jscall(sig) + '\n'
-    if settings.get('EMULATED_FUNCTION_POINTERS'):
+    if settings['EMULATED_FUNCTION_POINTERS']:
       args = ['a%d' % i for i in range(len(sig)-1)]
       full_args = ['x'] + args
-      table_access = 'FUNCTION_TABLE_' + sig
-      if settings['SIDE_MODULE']:
-        table_access = 'parentModule["' + table_access + '"]' # side module tables were merged into the parent, we need to access the global one
-      if settings['BINARYEN']:
-        # wasm uses a Table, which means we have function pointer emulation capabilities all the time, at no cost. just call the table
-        table_access = "Module['wasmTable']"
-        table_read = table_access + '.get(x)'
+      if settings['WASM']:
+        if settings['EMULATE_FUNCTION_POINTER_CASTS']:
+          # emulated function pointers in wasm use an internal i64-based ABI with a fixed number of arguments. we can't
+          # call into it directly because it returns an i64, which is an error for the VM. instead, we use dynCalls
+          dyn_call = "Module['asm']['dynCall_" + sig + "']"
+          asm_setup += '''
+function ftCall_%s(%s) {
+  return %s(%s);
+}
+''' % (sig, ', '.join(full_args), dyn_call, ', '.join(full_args))
+          # and we are done with this signature, continue
+          continue
+        else:
+          # otherwise, wasm emulated function pointers *without* emulated casts can just all
+          # into the table
+          table_access = "Module['wasmTable']"
+          table_read = table_access + '.get(x)'
       else:
+        table_access = 'FUNCTION_TABLE_' + sig
+        if settings['SIDE_MODULE']:
+          table_access = 'parentModule["' + table_access + '"]' # side module tables were merged into the parent, we need to access the global one
         table_read = table_access + '[x]'
       prelude = '''
   if (x < 0 || x >= %s.length) { Module.printErr("Function table mask error (out of range)"); %s ; abort(x) }''' % (table_access, get_function_pointer_error(sig, function_table_sigs, settings))
@@ -1227,9 +1244,9 @@ def create_basic_funcs(function_table_sigs, settings):
 
   for sig in function_table_sigs:
     basic_funcs.append('invoke_%s' % sig)
-    if settings.get('RESERVED_FUNCTION_POINTERS'):
+    if settings['RESERVED_FUNCTION_POINTERS']:
       basic_funcs.append('jsCall_%s' % sig)
-    if settings.get('EMULATED_FUNCTION_POINTERS'):
+    if settings['EMULATED_FUNCTION_POINTERS']:
       if not settings['BINARYEN']: # in wasm, emulated function pointers are just simple table calls
         basic_funcs.append('ftCall_%s' % sig)
   return basic_funcs
@@ -1266,13 +1283,18 @@ def create_exports(exported_implemented_functions, in_table, function_table_data
   exports = []
   for export in sorted(set(all_exported)):
     exports.append(quote(export) + ": " + export)
-  if settings['BINARYEN'] and settings['SIDE_MODULE']:
+  if settings['WASM'] and settings['SIDE_MODULE']:
     # named globals in side wasm modules are exported globals from asm/wasm
     for k, v in metadata['namedGlobals'].items():
       exports.append(quote('_' + str(k)) + ': ' + str(v))
     # aliases become additional exports
     for k, v in metadata['aliases'].items():
       exports.append(quote(str(k)) + ': ' + str(v))
+  # shared wasm emulated function pointer mode requires us to know the function pointer for
+  # each function. export fp$func => function pointer for func
+  if settings['WASM'] and settings['RELOCATABLE'] and settings['EMULATED_FUNCTION_POINTERS']:
+    for k, v in metadata['functionPointers'].items():
+      exports.append(quote('fp$' + str(k)) + ': ' + str(v))
   return '{ ' + ', '.join(exports) + ' }'
 
 
@@ -1337,7 +1359,7 @@ def create_receiving(function_table_data, function_tables_defs, exported_impleme
       table = table.replace('var ' + tableName, 'var ' + tableName + ' = Module["' + tableName + '"]')
       receiving += table + '\n'
 
-  if settings.get('EMULATED_FUNCTION_POINTERS'):
+  if settings['EMULATED_FUNCTION_POINTERS']:
     receiving += '\n' + function_tables_defs.replace('// EMSCRIPTEN_END_FUNCS\n', '') + '\n' + ''.join(['Module["dynCall_%s"] = dynCall_%s\n' % (sig, sig) for sig in function_table_data])
     if not settings['BINARYEN']:
       for sig in function_table_data.keys():
@@ -1366,7 +1388,8 @@ Module['NAMED_GLOBALS'] = NAMED_GLOBALS;
       named_globals += '''
 for (var named in NAMED_GLOBALS) {
   (function(named) {
-    Module['g$_' + named] = function() { return Module['_' + named] };
+    var func = Module['_' + named];
+    Module['g$_' + named] = function() { return func };
   })(named);
 }
 '''
@@ -1420,6 +1443,13 @@ function emtStackRestore(x) {
   x = x | 0;
   EMTSTACKTOP = x;
 }
+function getEmtStackMax() {
+  return EMT_STACK_MAX | 0;
+}
+function setEmtStackMax(x) {
+  x = x | 0;
+  EMT_STACK_MAX = x;
+}
 ''' if settings['EMTERPRETIFY_ASYNC'] else '') + '''
 function setThrew(threw, value) {
   threw = threw|0;
@@ -1705,33 +1735,26 @@ HEAP_TYPE_INFOS = [
 ]
 
 
-def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_engine=None,
-                          temp_files=None, DEBUG=None):
+def emscript_wasm_backend(infile, settings, outfile, libraries, compiler_engine,
+                          temp_files, DEBUG):
   # Overview:
   #   * Run LLVM backend to emit .s
   #   * Run Binaryen's s2wasm to generate WebAssembly.
   #   * We may also run some Binaryen passes here.
 
-  if libraries is None: libraries = []
+  if shared.Settings.EXPERIMENTAL_USE_LLD:
+    wasm, metadata = build_wasm_lld(temp_files, infile, outfile, settings, DEBUG)
+  else:
+    wasm, metadata = build_wasm(temp_files, infile, outfile, settings, DEBUG)
 
-  wast = build_wasm(temp_files, infile, outfile, settings, DEBUG)
+  # optimize syscalls
+
+  optimize_syscalls(metadata['declares'], settings, DEBUG)
 
   # js compiler
 
   if DEBUG: logging.debug('emscript: js compiler glue')
 
-  # Integrate info from backend
-
-  output = open(wast).read()
-  parts = output.split('\n;; METADATA:')
-  assert len(parts) == 2
-  metadata_raw = parts[1]
-  parts = output = None
-
-  if DEBUG: logging.debug("METAraw %s", metadata_raw)
-  metadata = create_metadata_wasm(metadata_raw, wast)
-  if DEBUG: logging.debug(repr(metadata))
-
   update_settings_glue(settings, metadata)
 
   if DEBUG: t = time.time()
@@ -1746,7 +1769,7 @@ def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_en
 
   # memory and global initializers
 
-  global_initializers = str(', '.join(['{ func: function() { %s() } }' % i for i in metadata['initializers']]))
+  global_initializers = ', '.join('{ func: function() { %s() } }' % i for i in metadata['initializers'])
 
   staticbump = metadata['staticBump']
   while staticbump % 16 != 0: staticbump += 1
@@ -1765,12 +1788,20 @@ def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_en
   exported_implemented_functions = create_exported_implemented_functions_wasm(pre, forwarded_json, metadata, settings)
 
   asm_consts, asm_const_funcs = create_asm_consts_wasm(forwarded_json, metadata)
-  pre = pre.replace('// === Body ===', '// === Body ===\n' + '\nvar ASM_CONSTS = [' + ',\n '.join(asm_consts) + '];\n' + '\n'.join(asm_const_funcs) + '\n')
+  em_js_funcs = create_em_js(forwarded_json, metadata)
+  pre = pre.replace(
+    '// === Body ===',
+    ('// === Body ===\n\nvar ASM_CONSTS = [' +
+      ',\n '.join(asm_consts) + '];\n' +
+      asstr('\n'.join(asm_const_funcs)) +
+      '\n'.join(em_js_funcs) + '\n'))
 
   outfile.write(pre)
   pre = None
 
-  invoke_funcs = read_wast_invoke_imports(wast)
+  invoke_funcs = metadata.get('invokeFuncs', [])
+  # List of function signatures used in jsCall functions, e.g.['v', 'vi']
+  jscall_sigs = metadata.get('jsCallFuncType', [])
 
   try:
     del forwarded_json['Variables']['globals']['_llvm_global_ctors'] # not a true variable
@@ -1778,11 +1809,13 @@ def emscript_wasm_backend(infile, settings, outfile, libraries=None, compiler_en
     pass
 
   # sent data
-  sending = create_sending_wasm(invoke_funcs, forwarded_json, metadata, settings)
+  sending = create_sending_wasm(invoke_funcs, jscall_sigs, forwarded_json,
+                                metadata, settings)
   receiving = create_receiving_wasm(exported_implemented_functions, settings)
 
   # finalize
-  module = create_module_wasm(sending, receiving, invoke_funcs, exported_implemented_functions, settings)
+  module = create_module_wasm(sending, receiving, invoke_funcs, jscall_sigs,
+                              exported_implemented_functions, settings)
 
   write_output_file(outfile, post, module)
   module = None
@@ -1804,38 +1837,105 @@ def build_wasm(temp_files, infile, outfile, settings, DEBUG):
 
     assert shared.Settings.BINARYEN_ROOT, 'need BINARYEN_ROOT config set so we can use Binaryen s2wasm on the backend output'
     basename = shared.unsuffixed(outfile.name)
-    wast = basename + '.wast'
     wasm = basename + '.wasm'
-    s2wasm_args = create_s2wasm_args(temp_s)
+    metadata_file = basename + '.metadata'
+    s2wasm_args = create_s2wasm_args(temp_s, wasm)
+    if settings['DEBUG_LEVEL'] >= 2 or settings['PROFILING_FUNCS']:
+      s2wasm_args += ['-g']
+      if settings['DEBUG_LEVEL'] >= 4:
+        s2wasm_args += ['--source-map=' + wasm + '.map']
+        if not settings['SOURCE_MAP_BASE']:
+          logging.warn("Wasm source map won't be usable in a browser without --source-map-base")
+        else:
+          s2wasm_args += ['--source-map-url=' + settings['SOURCE_MAP_BASE'] + os.path.basename(settings['WASM_BINARY_FILE']) + '.map']
     if DEBUG:
       logging.debug('emscript: binaryen s2wasm: ' + ' '.join(s2wasm_args))
       t = time.time()
       #s2wasm_args += ['--debug']
-    shared.check_call(s2wasm_args, stdout=open(wast, 'w'))
-    # Also convert wasm text to binary
-    wasm_as_args = [os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 'wasm-as'),
-                    wast, '-o', wasm]
-    if settings['DEBUG_LEVEL'] >= 2 or settings['PROFILING_FUNCS']:
-      wasm_as_args += ['-g']
-      if settings['DEBUG_LEVEL'] >= 4:
-        wasm_as_args += ['--source-map=' + wasm + '.map']
-        if not settings['SOURCE_MAP_BASE']:
-          logging.warn("Wasm source map won't be usable in a browser without --source-map-base")
-        else:
-          wasm_as_args += ['--source-map-url=' + settings['SOURCE_MAP_BASE'] + os.path.basename(settings['WASM_BINARY_FILE']) + '.map']
-    logging.debug('  emscript: binaryen wasm-as: ' + ' '.join(wasm_as_args))
-    shared.check_call(wasm_as_args)
+    shared.check_call(s2wasm_args, stdout=open(metadata_file, 'w'))
+
+  metadata = create_metadata_wasm(open(metadata_file).read(), DEBUG)
 
   if DEBUG:
     logging.debug('  emscript: binaryen s2wasm took %s seconds' % (time.time() - t))
     t = time.time()
-    shutil.copyfile(wast, os.path.join(shared.CANONICAL_TEMP_DIR, 'emcc-s2wasm-output.wast'))
-  return wast
+  return wasm, metadata
 
 
-def create_metadata_wasm(metadata_raw, wast):
+def build_wasm_lld(temp_files, infile, outfile, settings, DEBUG):
+  assert shared.Settings.BINARYEN_ROOT, 'need BINARYEN_ROOT config set so we can use Binaryen tools on the backend output'
+  wasm_emscripten_finalize = os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 'wasm-emscripten-finalize')
+  wasm_as = os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 'wasm-as')
+  wasm_dis = os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 'wasm-dis')
+
+  def debug_copy(src, dst):
+    if DEBUG:
+      shutil.copyfile(src, os.path.join(shared.CANONICAL_TEMP_DIR, dst))
+      if src[-2:] == '.o' or src[-5:] == '.wasm':
+        tmp = dst + '.wast'
+        shared.check_call([wasm_dis, src, '-o', tmp])
+        shutil.copyfile(tmp, os.path.join(shared.CANONICAL_TEMP_DIR, tmp))
+
+  with temp_files.get_file('.wb.o') as temp_o:
+    backend_args = create_backend_args_wasm(infile, temp_o, settings)
+    if DEBUG:
+      logging.debug('emscript: llvm wasm backend: ' + ' '.join(backend_args))
+      t = time.time()
+    shared.check_call(backend_args)
+    if DEBUG:
+      logging.debug('  emscript: llvm wasm backend took %s seconds' % (time.time() - t))
+      t = time.time()
+    debug_copy(temp_o, 'emcc-llvm-backend-output.o')
+
+    basename = shared.unsuffixed(outfile.name)
+    wasm = basename + '.wasm'
+    metadata_file = basename + '.metadata'
+    base_wasm = basename + '.lld.wasm'
+
+    libc_rt_lib = shared.Cache.get('wasm_libc_rt.a', wasm_rt_fail('wasm_libc_rt.a'), 'a')
+    compiler_rt_lib = shared.Cache.get('wasm_compiler_rt.a', wasm_rt_fail('wasm_compiler_rt.a'), 'a')
+    cmd = [shared.LLD, '-flavor', 'wasm',
+      '-z', 'stack-size=%s' % settings['TOTAL_STACK'],
+      '--global-base=%s' % shared.Settings.GLOBAL_BASE,
+      '--initial-memory=%s' % shared.Settings.TOTAL_MEMORY,
+      temp_o, libc_rt_lib, compiler_rt_lib,
+      '-o', base_wasm,
+      '--entry=main',
+      '--allow-undefined',
+      '--import-memory',
+      '--export', '__wasm_call_ctors']
+    for export in shared.expand_response(settings['EXPORTED_FUNCTIONS']):
+      cmd += ['--export', export[1:]] # Strip the leading underscore
+    shared.check_call(cmd)
+
+    if DEBUG:
+      logging.debug('  emscript: lld took %s seconds' % (time.time() - t))
+      t = time.time()
+    debug_copy(base_wasm, 'base_wasm.wasm')
+
+    shared.check_call([wasm_emscripten_finalize, base_wasm, '-o', wasm,
+                       '--global-base=%s' % shared.Settings.GLOBAL_BASE,
+                       ('--emscripten-reserved-function-pointers=%d' %
+                        shared.Settings.RESERVED_FUNCTION_POINTERS)],
+                      stdout=open(metadata_file, 'w'))
+
+    metadata = create_metadata_wasm(open(metadata_file).read(), DEBUG)
+
+  return wasm, metadata
+
+
+def read_metadata_wast(wast, DEBUG):
+  output = open(wast).read()
+  parts = output.split('\n;; METADATA:')
+  assert len(parts) == 2
+  metadata_raw = parts[1]
+  return create_metadata_wasm(metadata_raw, DEBUG)
+
+
+def create_metadata_wasm(metadata_raw, DEBUG):
+  if DEBUG: logging.debug("METAraw %s", metadata_raw)
   metadata = load_metadata(metadata_raw)
-  add_metadata_from_wast(metadata, wast)
+  if DEBUG: logging.debug(repr(metadata))
   return metadata
 
 
@@ -1853,7 +1953,7 @@ def create_exported_implemented_functions_wasm(pre, forwarded_json, metadata, se
     if key in all_exported_functions or export_all or (export_bindings and key.startswith('_emscripten_bind')):
       exported_implemented_functions.add(key)
 
-  if settings['ASSERTIONS'] and settings.get('ORIGINAL_EXPORTED_FUNCTIONS'):
+  if settings['ASSERTIONS'] and settings['ORIGINAL_EXPORTED_FUNCTIONS']:
     original_exports = get_original_exported_functions(settings)
     for requested in original_exports:
       # check if already implemented
@@ -1865,6 +1965,7 @@ def create_exported_implemented_functions_wasm(pre, forwarded_json, metadata, se
 
   return exported_implemented_functions
 
+
 def create_asm_consts_wasm(forwarded_json, metadata):
   asm_consts = [0]*len(metadata['asmConsts'])
   all_sigs = []
@@ -1872,40 +1973,64 @@ def create_asm_consts_wasm(forwarded_json, metadata):
     const = asstr(v[0])
     sigs = v[1]
     const = trim_asm_const_body(const)
-    const = '{ ' + const + ' }'
     args = []
-    arity = max(map(len, sigs)) - 1
+    max_arity = 16
+    arity = 0
+    for i in range(max_arity):
+      if ('$' + str(i)) in const:
+        arity = i + 1
     for i in range(arity):
       args.append('$' + str(i))
-    const = 'function(' + ', '.join(args) + ') ' + const
+    const = 'function(' + ', '.join(args) + ') {' + const + '}'
     asm_consts[int(k)] = const
     all_sigs += sigs
 
   asm_const_funcs = []
   for sig in set(all_sigs):
     forwarded_json['Functions']['libraryFunctions']['_emscripten_asm_const_' + sig] = 1
-    args = ['a%d' % i for i in range(len(sig)-1)]
-    all_args = ['code'] + args
     asm_const_funcs.append(r'''
-function _emscripten_asm_const_%s(%s) {
-return ASM_CONSTS[code](%s);
-}''' % (asstr(sig), ', '.join(all_args), ', '.join(args)))
-
+function _emscripten_asm_const_%s(code, sig_ptr, argbuf) {
+  var sig = AsciiToString(sig_ptr);
+  var args = [];
+  var align_to = function(ptr, align) {
+    return (ptr+align-1) & ~(align-1);
+  };
+  var buf = argbuf;
+  for (var i = 0; i < sig.length; i++) {
+    var c = sig[i];
+    if (c == 'd' || c == 'f') {
+      buf = align_to(buf, 8);
+      args.push(HEAPF64[(buf >> 3)]);
+      buf += 8;
+    } else if (c == 'i') {
+      buf = align_to(buf, 4);
+      args.push(HEAPU32[(buf >> 2)]);
+      buf += 4;
+    }
+  }
+  return ASM_CONSTS[code].apply(null, args);
+}''' % sig)
   return asm_consts, asm_const_funcs
 
 
-def read_wast_invoke_imports(wast):
-  invoke_funcs = []
-  for line in open(wast).readlines():
-    if line.strip().startswith('(import '):
-      parts = line.split()
-      func_name = parts[2][1:-1]
-      if func_name.startswith('invoke_'):
-        invoke_funcs.append(func_name)
-  return invoke_funcs
+def create_em_js(forwarded_json, metadata):
+  em_js_funcs = []
+  separator = '<::>'
+  for name, raw in metadata.get('emJsFuncs', {}).items():
+    parts = raw.split(separator)
+    assert len(parts) >= 2
+    args, body = parts[0], separator.join(parts[1:])
+    args = args[1:-1].split(',')
+    arg_names = [arg.split()[-1] for arg in args if arg]
+    func = 'function {}({}){}'.format(name, ','.join(arg_names), body)
+    em_js_funcs.append(func)
+    forwarded_json['Functions']['libraryFunctions'][name] = 1
+
+  return em_js_funcs
 
 
-def create_sending_wasm(invoke_funcs, forwarded_json, metadata, settings):
+def create_sending_wasm(invoke_funcs, jscall_sigs, forwarded_json, metadata,
+                        settings):
   basic_funcs = ['abort', 'assert', 'enlargeMemory', 'getTotalMemory']
   if settings['ABORTING_MALLOC']:
     basic_funcs += ['abortOnCannotGrowMemory']
@@ -1922,7 +2047,10 @@ def create_sending_wasm(invoke_funcs, forwarded_json, metadata, settings):
   implemented_functions = set(metadata['implementedFunctions'])
   global_funcs = list(set([key for key, value in forwarded_json['Functions']['libraryFunctions'].items() if value != 2]).difference(set(global_vars)).difference(implemented_functions))
 
-  send_items = basic_funcs + invoke_funcs + global_funcs + basic_vars + global_vars
+  jscall_funcs = ['jsCall_' + sig for sig in jscall_sigs]
+
+  send_items = (basic_funcs + invoke_funcs + jscall_funcs + global_funcs +
+                basic_vars + global_vars)
   def math_fix(g):
     return g if not g.startswith('Math_') else g.split('_')[1]
   return '{ ' + ', '.join(['"' + math_fix(s) + '": ' + s for s in send_items]) + ' }'
@@ -1948,9 +2076,11 @@ return real_''' + asmjs_mangle(s) + '''.apply(null, arguments);
   return receiving
 
 
-def create_module_wasm(sending, receiving, invoke_funcs, exported_implemented_functions, settings):
+def create_module_wasm(sending, receiving, invoke_funcs, jscall_sigs,
+                       exported_implemented_functions, settings):
   access_quote = access_quoter(settings)
   invoke_wrappers = create_invoke_wrappers(invoke_funcs)
+  jscall_funcs = create_jscall_funcs(jscall_sigs)
 
   the_global = '{}'
 
@@ -1972,16 +2102,12 @@ var asm = Module['asm'](%s, %s, buffer);
      'Module' + access_quote('asmLibraryArg'),
      receiving)]
 
-  # wasm backend stack goes down, and is stored in the first global var location
   module.append('''
-STACKTOP = STACK_BASE + TOTAL_STACK;
-STACK_MAX = STACK_BASE;
-HEAP32[%d >> 2] = STACKTOP;
 var stackAlloc = Module['_stackAlloc'];
 var stackSave = Module['_stackSave'];
 var stackRestore = Module['_stackRestore'];
 var establishStackSpace = Module['establishStackSpace'];
-''' % shared.Settings.GLOBAL_BASE)
+''')
 
   # some runtime functionality may not have been generated in
   # the wasm; provide a JS shim for it
@@ -1990,13 +2116,18 @@ var establishStackSpace = Module['establishStackSpace'];
       module.append('var %s;\n' % name)
 
   module.append(invoke_wrappers)
+  module.append(jscall_funcs)
   return module
 
 def create_backend_args_wasm(infile, temp_s, settings):
   backend_compiler = os.path.join(shared.LLVM_ROOT, 'llc')
-  args = [backend_compiler, infile, '-march=wasm32', '-filetype=asm',
+  args = [backend_compiler, infile, '-mtriple={}'.format(shared.WASM_TARGET),
                   '-asm-verbose=false',
                   '-o', temp_s]
+  if settings['EXPERIMENTAL_USE_LLD']:
+    args += ['-filetype=obj']
+  else:
+    args += ['-filetype=asm']
   args += ['-thread-model=single'] # no threads support in backend, tell llc to not emit atomics
   # disable slow and relatively unimportant optimization passes
   args += ['-combiner-global-alias-analysis=false']
@@ -2012,21 +2143,23 @@ def create_backend_args_wasm(infile, temp_s, settings):
   args += ['-enable-emscripten-sjlj']
   return args
 
+def wasm_rt_fail(archive_file):
+  def wrapped():
+    raise Exception('Expected {} to already be built'.format(archive_file))
+  return wrapped
 
-def create_s2wasm_args(temp_s):
-  def wasm_rt_fail(archive_file):
-    def wrapped():
-      raise Exception('Expected {} to already be built'.format(archive_file))
-    return wrapped
+def create_s2wasm_args(temp_s, wasm):
   compiler_rt_lib = shared.Cache.get('wasm_compiler_rt.a', wasm_rt_fail('wasm_compiler_rt.a'), 'a')
   libc_rt_lib = shared.Cache.get('wasm_libc_rt.a', wasm_rt_fail('wasm_libc_rt.a'), 'a')
 
   s2wasm_path = os.path.join(shared.Settings.BINARYEN_ROOT, 'bin', 's2wasm')
 
-  args = [s2wasm_path, temp_s, '--emscripten-glue']
+  args = [s2wasm_path, temp_s, '-o', wasm, '--emscripten-glue', '--emit-binary']
   args += ['--global-base=%d' % shared.Settings.GLOBAL_BASE]
   args += ['--initial-memory=%d' % shared.Settings.TOTAL_MEMORY]
   args += ['--allow-memory-growth'] if shared.Settings.ALLOW_MEMORY_GROWTH else []
+  args += ['--emscripten-reserved-function-pointers=%d' %
+           shared.Settings.RESERVED_FUNCTION_POINTERS]
   args += ['-l', libc_rt_lib]
   args += ['-l', compiler_rt_lib]
 
@@ -2050,6 +2183,8 @@ def load_metadata(metadata_raw):
     'maxGlobalAlign': 0,
     'initializers': [],
     'exports': [],
+    'emJsFuncs': {},
+    'invokeFuncs': [],
   }
 
   for k, v in metadata_json.items():
@@ -2064,47 +2199,6 @@ def load_metadata(metadata_raw):
   return metadata
 
 
-def add_metadata_from_wast(metadata, wast):
-  """Reads .wast file and adds metadata we can read from the code.
-
-  TODO: emit this metadata directly from s2wasm.
-  """
-  for line in open(wast).readlines():
-    line = line.strip()
-    if line.startswith('(import '):
-      parts = line.split()
-      # Don't include Invoke wrapper names (for asm.js-style exception handling)
-      # in metadata[declares], the invoke wrappers will be generated in
-      # this script later.
-      import_type = parts[3][1:]
-      import_name = parts[2][1:-1]
-      if import_type == 'memory':
-        continue
-      elif import_type == 'func':
-        if not import_name.startswith('invoke_'):
-          metadata['declares'].append(import_name)
-      elif import_type == 'global':
-        metadata['externs'].append('_' + import_name)
-      else:
-        assert False, 'Unhandled import type "%s"' % import_type
-    elif line.startswith('(func '):
-      parts = line.split()
-      func_name = parts[1][1:]
-      metadata['implementedFunctions'].append('_' + func_name)
-    elif line.startswith('(export '):
-      parts = line.split()
-      export_name = parts[1][1:-1]
-      export_type = parts[2][1:]
-      if export_type == 'func':
-        assert asmjs_mangle(export_name) not in metadata['exports']
-        metadata['exports'].append(export_name)
-      else:
-        assert False, 'Unhandled export type "%s"' % export_type
-
-  # we emit those ourselves
-  metadata['declares'] = [x for x in metadata['declares'] if not x.startswith('emscripten_asm_const')]
-
-
 def create_invoke_wrappers(invoke_funcs):
   """Asm.js-style exception handling: invoke wrapper generation."""
   invoke_wrappers = ''
@@ -2114,6 +2208,13 @@ def create_invoke_wrappers(invoke_funcs):
   return invoke_wrappers
 
 
+def create_jscall_funcs(sigs):
+  jscall_funcs = ''
+  for i, sig in enumerate(sigs):
+    jscall_funcs += '\n' + shared.JS.make_jscall(sig, i) + '\n'
+  return jscall_funcs
+
+
 def asmjs_mangle(name):
   """Mangle a name the way asm.js/JSBackend globals are mangled.
 
@@ -2126,11 +2227,6 @@ def asmjs_mangle(name):
   return '_' + ''.join(['_' if not c.isalnum() else c for c in name])
 
 
-if os.environ.get('EMCC_FAST_COMPILER') == '0':
-  logging.critical('Non-fastcomp compiler is no longer available, please use fastcomp or an older version of emscripten')
-  sys.exit(1)
-
-
 def normalize_line_endings(text):
   """Normalize to UNIX line endings.
 
@@ -2150,13 +2246,13 @@ def main(args, compiler_engine, cache, temp_files, DEBUG):
     settings[name] = asstr(value)
 
   # libraries
-  libraries = args.libraries[0].split(',') if len(args.libraries) > 0 else []
+  libraries = args.libraries[0].split(',') if len(args.libraries) else []
 
   settings.setdefault('STRUCT_INFO', shared.path_from_root('src', 'struct_info.compiled.json'))
-  struct_info = settings.get('STRUCT_INFO')
+  struct_info = settings['STRUCT_INFO']
 
-  if not os.path.exists(struct_info) and not settings.get('BOOTSTRAPPING_STRUCT_INFO') and not settings.get('ONLY_MY_CODE'):
-    if DEBUG: logging.debug('  emscript: bootstrapping struct info...')
+  if not os.path.exists(struct_info) and not settings['BOOTSTRAPPING_STRUCT_INFO'] and not settings['ONLY_MY_CODE']:
+    if DEBUG: logging.debug('  emscript: bootstrapping struct info to %s', struct_info)
     shared.Building.ensure_struct_info(struct_info)
     if DEBUG: logging.debug('  emscript: bootstrapping struct info complete')
 
@@ -2166,27 +2262,13 @@ def main(args, compiler_engine, cache, temp_files, DEBUG):
              temp_files=temp_files, DEBUG=DEBUG)
 
 
-def _main(args=None):
-  if args is None:
-    args = sys.argv[1:]
-
-  response_file = True
-  while response_file:
-    response_file = None
-    for index in range(len(args)):
-      if args[index][0] == '@':
-        # found one, loop again next time
-        response_file = True
-        response_file_args = read_response_file(args[index])
-        # slice in extra_args in place of the response file arg
-        args[index:index+1] = response_file_args
-        break
+def _main(args):
+  substitute_response_files(args)
 
   parser = argparse.ArgumentParser(
     usage='%(prog)s [-h] [-H HEADERS] [-o OUTFILE] [-c COMPILER_ENGINE] [-s FOO=BAR]* infile',
     description=('You should normally never use this! Use emcc instead. '
-                 'This is a wrapper around the JS compiler, converting .ll to .js.'),
-    epilog='')
+                 'This is a wrapper around the JS compiler, converting .ll to .js.'))
   parser.add_argument('-H', '--headers',
                     default=[],
                     action='append',
@@ -2239,7 +2321,9 @@ WARNING: You should normally never use this! Use emcc instead.
   ''')
 
   if len(positional) != 1:
-    raise RuntimeError('Must provide exactly one positional argument. Got ' + str(len(positional)) + ': "' + '", "'.join(positional) + '"')
+    logging.error('Must provide exactly one positional argument. Got ' + str(len(positional)) + ': "' + '", "'.join(positional) + '"')
+    return 1
+
   keywords.infile = os.path.abspath(positional[0])
   if isinstance(keywords.outfile, (type(u''), bytes)):
     keywords.outfile = open(keywords.outfile, 'w')
@@ -2269,7 +2353,7 @@ WARNING: You should normally never use this! Use emcc instead.
     temp_files=temp_files,
     DEBUG=DEBUG,
   ))
+  return 0
 
 if __name__ == '__main__':
-  _main()
-  sys.exit(0)
+  sys.exit(_main(sys.argv[1:]))
